@@ -16,13 +16,14 @@
 """cubicweb-worker entity's classes"""
 import socket
 
-from contextlib import contextmanager
 from logging import getLogger
 from warnings import warn
 
-from cubicweb import ValidationError, UnknownEid, set_log_methods
-from cubicweb.predicates import is_instance
-from cubicweb.view import EntityAdapter
+from logilab.common.registry import RegistrableObject, ObjectNotFound
+from logilab.common.deprecation import class_renamed
+
+from cubicweb import ValidationError, set_log_methods
+from cubicweb.predicates import yes
 from cubicweb.entities import AnyEntity, fetch_config
 
 from cubes.worker import Aborted
@@ -39,7 +40,7 @@ class CWWorker(AnyEntity, UtilsHolder):
                 'S name IN ("task_pending", "task_assigned"), X eid %(e)s')
 
     def dc_title(self):
-        return ('%s (hostname: %s last seen: %s)' %
+        return (u'%s (hostname: %s last seen: %s)' %
                 (self.eid, socket.getfqdn(), self.last_ping))
 
     def get_load(self):
@@ -91,7 +92,7 @@ class CWWorker(AnyEntity, UtilsHolder):
         when multiple worker Race for the same Task."""
         self.commit_transition(task, 'task_acquire', lenient=True) # should fail in case of a race condition
         task = self._cw.entity_from_eid(task.eid)
-        task.set_relations(done_by=self)
+        task.cw_set(done_by=self)
         self.info('will process task %s', task.dc_long_title())
 
     def _handle_task_error(self, session, exc, task):
@@ -140,13 +141,17 @@ class CWWorker(AnyEntity, UtilsHolder):
                      'Use a Worker adapter instead.',
                      DeprecationWarning)
             else:
-                adapter = task.cw_adapt_to(task.operation)
-                if not adapter:
-                    self.warning('Bailing out: could not find adapter for %s', task.operation)
-                    return self._handle_task_error(session,
-                                                   Exception('no adapter for %s' % task.operation),
-                                                   task)
-                method = adapter.perform_task
+                try:
+                    performer = session.vreg['worker.performer'].select(task.operation,
+                                                                        session,
+                                                                        entity=task)
+                except ObjectNotFound:
+                    performer = task.cw_adapt_to(task.operation)
+                    if performer is None:
+                        self.warning('Bailing out: could not find adapter for %s', task.operation)
+                        exc = Exception('no adapter for %s' % task.operation)
+                        return self._handle_task_error(session, exc, task)
+                method = performer.perform_task
             try:
                 # we give the session and forget about it
                 # hence it may be in an unpredictable state
@@ -160,16 +165,25 @@ class CWWorker(AnyEntity, UtilsHolder):
                 return self.commit_transition(task, 'task_complete', result)
 
 
-class WorkerAdapter(EntityAdapter):
+class Performer(RegistrableObject):
     # the __regid__ will be transmitted as the operation of the workertask entity
+    __registry__ = 'worker.performer'
     __regid__ = '<task.operation>'
-    __select__ = is_instance('CWWorkerTask')
+    __select__ = yes()
     __abstract__ = True
 
+    def __init__(self, session, entity=None):
+        # this is to remain compatible with the previous
+        # adapter-based implementation
+        self._cw = session
+        self.entity = entity
+
     def perform_task(self, session, task):
         """ real body/implementation of the task """
         raise NotImplementedError
 
+WorkerAdapter = class_renamed('WorkerAdapter', Performer,
+                              'The WorkerAdapter is replaced with Performer')
 
 
 class CWWorkerTask(AnyEntity):
@@ -188,3 +202,4 @@ class CWWorkerTask(AnyEntity):
 
 
 set_log_methods(CWWorker, getLogger('cubicweb.worker'))
+set_log_methods(Performer, getLogger('cubicweb.worker'))
