@@ -11,9 +11,9 @@ def getrepo(obj):
     """
     if not isinstance(obj, Repository):
         try:
-            repo = obj._cw.vreg.config.repository()
-        except AttributeError: # we are not running an all-in-one instance
             repo = obj._cw.repo
+        except AttributeError:
+            repo = obj._cw.vreg.config.repository()
     else:
         repo = obj
     assert repo
